#! /usr/bin/env python

"""
   Program:   EQ Alert
   File Name: eqa_state.py
   Copyright (C) 2019 Michael Geitz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Parse and react to eqemu logs
"""

class EQA_State:
  def __init__(self, char, chars, zone, loc, direction, afk, server):
    self.char = char
    self.chars = chars
    self.zone = zone
    self.loc = loc
    self.direction = direction
    self.afk = afk
    self.server = server

  def set_char(self, char):
    self.char = char

  def set_chars(self, chars):
    self.chars = chars

  def set_zone(self, zone):
    self.zone = zone

  def set_loc(self, loc):
    self.loc = loc

  def set_direction(self, direction):
    self.direction = direction

  def set_afk(self, afk):
    self.afk = afk

  def set_server(self, server):
    self.server = server



#! /usr/bin/env python

"""
   Program:   EQ Alert
   File Name: eqa_struct.py
   Copyright (C) 2019 Michael Geitz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from collections import namedtuple

global display
global sound
global message

message = namedtuple('data', ['timestamp', 'type', 'tx', 'rx', 'payload'])
display = namedtuple('data', ['timestamp', 'type', 'screen', 'payload'])
sound = namedtuple('data', ['sound', 'payload'])



#! /usr/bin/env python

"""
   Program:   EQ Alert
   File Name: eqa_settings.py
   Copyright (C) 2019 Michael Geitz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys
import os
import logging
import time
import datetime
import yaml
import pwd
import grp
from collections import namedtuple

def usage():
  """Print some helpful things"""
  print('ya done goofed')
  sys.exit(2)


def timestamp():
  """Returns a neat little timestamp for things"""
  unixstamp = int(time.time())
  timestamp = datetime.datetime.fromtimestamp(int(unixstamp))\
      .strftime('%Y-%m-%d_%H:%M:%S')
  return str(timestamp)


def eqa_time():
  """Returns message timestamp HH:MM:SS.ff"""
  return datetime.datetime.now().strftime('%H:%M:%S.%f')[:-4]


def log(message):
  """Effectively just for timestamping all log messages"""
  logging.info('[' + timestamp() + ']: ' + str(message))


if __name__ == '__main__':
  main()



#! /usr/bin/env python

"""
   Program:   EQ Alert
   File Name: eqa_sound.py
   Copyright (C) 2019 Michael Geitz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import os
import time
import sys
import gtts
from playsound import playsound

import eqa.lib.eqa_struct as eqa_struct
import eqa.lib.eqa_settings as eqa_settings


def process(config, sound_q, exit_flag, cfg_reload):
  """
    Process: sound_q
    Produce: sound event
  """

  tmp_sound_file_path='/tmp/eqa/sound/'

  if not os.path.exists(tmp_sound_file_path):
    os.makedirs(tmp_sound_file_path)

  try:
    while not exit_flag.is_set() and not cfg_reload.is_set():
      time.sleep(0.001)
      if not sound_q.empty():
        sound_event = sound_q.get()
        sound_q.task_done()

        if sound_event.sound == "espeak":
          speak(sound_event.payload, tmp_sound_file_path)
        elif sound_event.sound == "alert":
          alert(config, sound_event.payload)
        else:
          speak(sound_event.payload, tmp_sound_file_path)
          display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', "[Malformed sound event] " + sound_event.sound))
  except Exception as e:
    eqa_settings.log('process_sound: Error on line ' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))
    sys.exit()

  sys.exit()


def pre_speak(phrase, sound_file_path):
  """Save spoken phrase"""
  try:
    if not os.path.exists(sound_file_path + phrase):
      tts = gtts.gTTS(text=phrase, lang='en')
      tts.save(sound_file_path + phrase + '.wav')
  except Exception as e:
    eqa_settings.log('pre_speak: Error on line ' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def speak(phrase, tmp_sound_file_path):
  """Playa spoken phrase"""
  try:
    if not os.path.exists(tmp_sound_file_path + phrase + '.wav'):
      tts = gtts.gTTS(text=phrase, lang='en')
      tts.save(tmp_sound_file_path + phrase + '.wav')
    play_sound(tmp_sound_file_path + phrase + '.wav')
  except Exception as e:
    eqa_settings.log('speak: Error on line ' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def alert(config, line_type):
  if not config["line"][line_type]["sound"] == "0":
    play_sound(config["settings"]["paths"]["sound"] + \
    config["settings"]["sounds"][config["line"][line_type]["sound"]])


def play_sound(sound):
  """Play the sound given"""
  try:
    playsound(sound)
  except KeyboardInterrupt:
    pass


if __name__ == '__main__':
  main()



#! /usr/bin/env python

"""
   Program:   EQ Alert
   File Name: eqa_keys.py
   Copyright (C) 2019 Michael Geitz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import curses
import time
import sys

import eqa.lib.eqa_settings as eqa_settings
import eqa.lib.eqa_struct as eqa_struct

def process(keyboard_q, system_q, display_q, sound_q, exit_flag, heal_parse, spell_parse, raid, chars):
  """
    Process: keyboard_q
    Produce: sound_q, display_q, system_q, heal_q, damage_q
  """

  key = ''
  page = 'events'
  settings = 'character'
  selected_char = 0

  while key != ord('q') and key != 27:
    try:
      # Get key
      time.sleep(0.001)
      if not keyboard_q.empty():
        key = keyboard_q.get()
        keyboard_q.task_done()

        # Handle resize event
        if key == curses.KEY_RESIZE:
          display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'draw', 'redraw', 'null'))

        # Handle tab keys
        if key == curses.KEY_F1:
          display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'draw', 'events', 'null'))
          page = 'events'
        elif key == curses.KEY_F2:
          display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'draw', 'state', 'null'))
          page = 'state'
        elif key == curses.KEY_F3:
          display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'draw', 'settings', 'null'))
          page = 'settings'
        elif key == curses.KEY_F4:
          display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'draw', 'help', 'null'))
          page = 'help'
        elif key == curses.KEY_F12:
          system_q.put(eqa_struct.message(eqa_settings.eqa_time(), 'system', 'reload_config', 'null', 'null'))

        # Events keys
        if page == 'events':
          if key == ord('c'):
            display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'clear', 'null'))
          elif key == ord('r'):
            if not raid.is_set():
              raid.set()
              display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', 'Raid mode enabled'))
              sound_q.put(eqa_struct.sound('espeak', 'Raid mode enabled'))
            elif raid.is_set():
              raid.clear()
              display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', 'Raid mode disabled'))
              sound_q.put(eqa_struct.sound('espeak', 'Raid mode disabled'))
            display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'draw', 'events', 'null'))

        # State keys
        elif page == 'state':
          pass

        # Settings keys
        elif page == 'settings':
            if settings == 'character':
              if key == curses.KEY_UP or key == ord('w'):
                if selected_char < len(chars) - 1:
                  selected_char += 1
                  display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'update', 'selected_char', selected_char))
              elif key == curses.KEY_DOWN or key == ord('s'):
                if selected_char > 0:
                  selected_char -= 1
                  display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'update', 'selected_char', selected_char))
              elif key == curses.KEY_RIGHT or key == ord('d'):
                display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'update', 'select_char', selected_char))
                display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'update', 'zone', 'unavailable'))
                system_q.put(eqa_struct.message(eqa_settings.eqa_time(), 'system', 'new_character', 'null', chars[selected_char]))
              elif key == ord(' '):
                pass
                # cycle to next setting

        # Help keys
        elif page == 'help':
          pass

    except Exception as e:
      eqa_settings.log('process keys: ' + str(e))
      eqa_settings.log('setting exit_flag')
      exit_flag.set()
      sys.exit()

  exit_flag.set()
  sys.exit()


def read(exit_flag, keyboard_q, stdscr):
  """
    Consume: keyboard events
    Produce: keyboard_q
  """

  key = ''
  try:
    while not exit_flag.is_set():
      key = stdscr.getch()
      keyboard_q.put(key)
    sys.exit()
  except Exception as e:
    eqa_settings.log('read keys: ' + str(e))
    sys.exit()


if __name__ == '__main__':
  main()



#! /usr/bin/env python

"""
   Program:   EQ Alert
   File Name: eqa_action.py
   Copyright (C) 2019 Michael Geitz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import datetime
import time
import sys

import eqa.lib.eqa_sound as eqa_sound
import eqa.lib.eqa_struct as eqa_struct
import eqa.lib.eqa_settings as eqa_settings
import eqa.lib.eqa_config as eqa_config


def process(action_q, system_q, display_q, sound_q, heal_q, damage_q, exit_flag,
            heal_parse, spell_parse, raid, cfg_reload, config, base_path):
  """
    Process: action_q
    Produce: sound_q, display_q, system_q, heal_q, damage_q
  """

  try:
    while not exit_flag.is_set() and not cfg_reload.is_set():
      time.sleep(0.001)
      if not action_q.empty():
        new_message = action_q.get()
        action_q.task_done()
        line_type = new_message.type
        line_time = new_message.timestamp
        line_tx = new_message.tx
        line_rx = new_message.rx
        check_line = new_message.payload
        check_line_list = new_message.payload.split(' ')

        # Line specific checks
        if line_type == "undetermined":
          undetermined_line(check_line, base_path)
        if line_type == "location":
          loc = [float(check_line_list[3].replace(',', '')),
                 float(check_line_list[4].replace(',', '')),
                 float(check_line_list[5].replace(',', ''))]
          system_q.put(eqa_struct.message(eqa_settings.eqa_time(), 'system', 'loc', 'null', loc))
        elif line_type == "direction":
          direction = check_line_list[-1].replace('.', '')
          system_q.put(eqa_struct.message(eqa_settings.eqa_time(), 'system', 'direction', 'null', direction))
        elif line_type.startswith("you_afk"):
          if line_type == "you_afk_on":
            display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', 'You are now AFK'))
            system_q.put(eqa_struct.message(eqa_settings.eqa_time(), 'system', 'afk', 'null', 'true'))
          elif line_type == "you_afk_off":
            display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', 'You are no longer AFK'))
            system_q.put(eqa_struct.message(eqa_settings.eqa_time(), 'system', 'afk', 'null', 'false'))
        elif line_type == "you_new_zone":
          nz_iter = 0
          current_zone = ""
          while check_line_list.index("entered") + nz_iter + 1 < len(check_line_list):
            current_zone += check_line_list[check_line_list.index("entered") + nz_iter + 1] + " "
            nz_iter += 1
          current_zone = current_zone[:-2]
          current_zone = current_zone.rstrip('.')
          sound_q.put(eqa_struct.sound('espeak', current_zone))
          display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'update', 'zone', current_zone))
          system_q.put(eqa_struct.message(eqa_settings.eqa_time(), 'system', 'zone', 'null', current_zone))
          if current_zone not in config["zones"].keys():
            eqa_config.add_zone(current_zone, base_path)
          elif current_zone in config["zones"].keys() and not raid.is_set():
            if config["zones"][current_zone] == "raid":
              raid.set()
              display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', 'Raid mode auto-enabled'))
              sound_q.put(eqa_struct.sound('espeak', 'Raid mode enabled'))
          elif current_zone in config["zones"].keys() and raid.is_set():
            if config["zones"][current_zone] != "raid":
              raid.clear()
              display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', 'Raid mode auto-disabled'))
              sound_q.put(eqa_struct.sound('espeak', "Raid mode disabled"))

        # If line_type is a parsable type
        if line_type in config["line"].keys():
          # If line_type is parsed for as true
          if config["line"][line_type]["reaction"] == "true":
            for keyphrase, value in config["line"][line_type]["alert"].items():
              if str(keyphrase).lower() in check_line and value == "true":
                sound_q.put(eqa_struct.sound('alert', line_type))
                display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', line_type + ': ' + check_line[0:65]))
              elif str(keyphrase).lower() in check_line and value == "raid" and raid.is_set():
                if keyphrase == 'assist' or keyphrase == 'rampage':
                  payload = keyphrase + ' on ' + check_line_list[0]
                else:
                  payload = keyphrase
                sound_q.put(eqa_struct.sound('espeak', payload))
                display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', line_type + ': ' + check_line[0:65]))

          # Or if line_type is parsed for as all
          elif config["line"][line_type]["reaction"] == "all":
            # Heal parse
            if heal_parse.is_set() and line_type == "you_healed":
              heal_q.put(eqa_struct.heal(datetime.datetime.now(), 'heal', 'you', check_line_list[3], check_line_list[5]))

            # Spell damage parse
            elif spell_parse.is_set() and line_type == "spell_damage":
              length = check_line_list.index('was')
              sp_iter = 0
              name = ""
              while sp_iter < length:
                name = name + check_line_list[sp_iter] + " "
                sp_iter += 1
              damaged_q.put(eqa_struct.heal(datetime.datetime.now(), 'spell', 'null', name, check_line_list[-4]))

            # DoT damage parse
            elif spell_parse.is_set() and line_type == "dot_damage":
              length = check_line_list.index('has')
              dp_iter = 0
              name = ""
              damage = int(check_line_list[length + 2])
              while dp_iter < length:
                name = name + check_line_list[dp_iter] + " "
                dp_iter += 1
              damaged_q.put(eqa_struct.heal(datetime.datetime.now(), 'dot', 'null', name, damage))

            # Notify on all other all alerts
            else:
              sound_q.put(eqa_struct.sound('alert', line_type))
              display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', line_type + ': ' + check_line[0:65]))

          # Or if line_type is parsed for as a spoken alert
          elif config["line"][line_type]["reaction"] == "speak":
            display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', check_line))
            sound_q.put(eqa_struct.sound('espeak', check_line))

          # For triggers requiring all line_types
          if config["line"]["all"]["reaction"] == "true":
            for keyphrase, value in config["alert"]["all"].items():
              if keyphrase in check_line:
                sound_q.put(eqa_struct.sound('alert', line_type))
                display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', line_type + ': ' + check_line[0:65]))

        # If line_type is not a parsable type
        else:
          eqa_config.add_type(line_type, base_path)
          display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', 'added: ' + line_type))
          system_q.put(eqa_struct.message(eqa_settings.eqa_time(), 'system', 'reload_config', 'null', 'null'))

  except Exception as e:
    eqa_settings.log('process action: Error on line ' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' +  str(e))

  sys.exit()


def undetermined_line(line, base_path):
  """Temp function to log undetermined log lines"""
  f = open(base_path + 'log/undetermined.txt', 'a')
  f.write(line + '\n')
  f.close()


if __name__ == '__main__':
  main()



#! /usr/bin/env python

"""
   Program:   EQ Alert
   File Name: eqa_curses.py
   Copyright (C) 2019 Michael Geitz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import curses
import os
import time
import sys

import eqa.lib.eqa_struct as eqa_struct
import eqa.lib.eqa_state as eqa_state
import eqa.lib.eqa_settings as eqa_settings

def display(stdscr, display_q, state, raid, exit_flag):
  """
    Process: display_q
    Produce: display event
  """
  events = []
  page = 'events'
  setting = 'character'
  selected_char = 0

  try:
    while not exit_flag.is_set():
      time.sleep(0.001)
      if not display_q.empty():
        display_event = display_q.get()
        display_q.task_done()

        # Display Var Update
        if display_event.type == 'update':
          if display_event.screen == 'setting':
            setting = display_event.payload
            draw_page(stdscr, page, events, state, setting, selected_char, raid)
          elif display_event.screen == 'selected_char':
            selected_char = display_event.payload
            draw_page(stdscr, page, events, state, setting, selected_char, raid)
          elif display_event.screen == 'select_char':
            selected_char = display_event.payload
            state.char = state.chars[selected_char]
            draw_page(stdscr, page, events, state, setting, selected_char, raid)
          elif display_event.screen == 'zone':
            zone = display_event.payload
            draw_page(stdscr, page, events, state, setting, selected_char, raid)
          elif display_event.screen == 'char':
            state.char = display_event.payload
            draw_page(stdscr, page, events, state, setting, selected_char, raid)

        # Display Draw
        elif display_event.type == 'draw':
          if display_event.screen != 'redraw':
            page = display_event.screen
          draw_page(stdscr, page, events, state, setting, selected_char, raid)

        # Draw Update
        elif display_event.type == 'event':
          if display_event.screen == 'events':
            events.append(display_event)
            if page == 'events':
              draw_page(stdscr, page, events, state, setting, selected_char, raid)
          elif display_event.screen == 'clear':
            events = []
            draw_page(stdscr, page, events, state, setting, selected_char, raid)



  except Exception as e:
    eqa_settings.log('display: Error on line ' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))

  sys.exit()


def draw_page(stdscr, page, events, state, setting, selected_char, raid):
  y, x = stdscr.getmaxyx()
  try:
    if x >= 80 and y >= 40:
      if page == 'events':
        draw_events_frame(stdscr, state.char, state.zone, events)
      elif page == 'state':
        draw_state(stdscr, state, raid)
      elif page == 'settings':
        draw_settings(stdscr, state, setting, selected_char)
      elif page == 'help':
        draw_help(stdscr)
    else:
      draw_toosmall(stdscr)
  except Exception as e:
    eqa_settings.log('draw_page: Error on line ' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def init(state):
  """Create new stdscr in terminal"""
  stdscr = curses.initscr()
  os.system('setterm -cursor off')
  curses.start_color()
  curses.use_default_colors()
  curses.noecho()
  curses.cbreak()
  stdscr.keypad(1)
  stdscr.timeout(100)
  curses.init_pair(1, curses.COLOR_WHITE, -1)     # Title
  curses.init_pair(2, curses.COLOR_YELLOW, -1)    # Header
  curses.init_pair(3, curses.COLOR_CYAN, -1)      # Subtext
  curses.init_pair(4, curses.COLOR_MAGENTA, -1)   # Highlight
  curses.init_pair(5, curses.COLOR_GREEN, -1)     # Dunno
  draw_events_frame(stdscr, state.char, state.zone, [])
  return stdscr


def close_screens(stdscr):
  """Terminate stdscr"""
  os.system('setterm -cursor on')
  curses.nocbreak()
  stdscr.keypad(0)
  curses.echo()
  curses.endwin()

def draw_tabs(stdscr, tab):
  """Draw the tabs, duh"""
  y, x = stdscr.getmaxyx()
  center_y = int(y / 2)
  center_x = int(x / 2)

  # Bottom of tabs
  stdscr.addch(2, 0, curses.ACS_LTEE)
  stdscr.addch(2, x - 1, curses.ACS_RTEE)
  for c in range (x - 2):
    stdscr.addch(2, c + 1, curses.ACS_HLINE)

# Events tab
  stdscr.addstr(1, 2, "F1", curses.color_pair(3))
  stdscr.addstr(1, 4, ":", curses.color_pair(1))
  if tab == 'events':
    stdscr.addstr(1, 6, "events", curses.color_pair(4))
  else:
    stdscr.addstr(1, 6, "events", curses.color_pair(2))
  stdscr.addch(0, 13, curses.ACS_TTEE)
  stdscr.addch(1, 13, curses.ACS_VLINE)
  stdscr.addch(2, 13, curses.ACS_BTEE)

  # State tab
  stdscr.addstr(1, 15, "F2", curses.color_pair(3))
  stdscr.addstr(1, 17, ":", curses.color_pair(1))
  if tab == 'state':
    stdscr.addstr(1, 19, "state", curses.color_pair(4))
  else:
    stdscr.addstr(1, 19, "state", curses.color_pair(2))
  stdscr.addch(0, 25, curses.ACS_TTEE)
  stdscr.addch(1, 25, curses.ACS_VLINE)
  stdscr.addch(2, 25, curses.ACS_BTEE)

  # Settings tab
  stdscr.addstr(1, x - 25, "F3", curses.color_pair(3))
  stdscr.addstr(1, x - 23, ":", curses.color_pair(1))
  if tab == 'settings':
    stdscr.addstr(1, x - 21, "settings", curses.color_pair(4))
  else:
    stdscr.addstr(1, x - 21, "settings", curses.color_pair(2))
  stdscr.addch(0, x - 27, curses.ACS_TTEE)
  stdscr.addch(1, x - 27, curses.ACS_VLINE)
  stdscr.addch(2, x - 27, curses.ACS_BTEE)

  # Help tab
  stdscr.addstr(1, x - 10, "F4", curses.color_pair(3))
  stdscr.addstr(1, x - 8, ":", curses.color_pair(1))
  if tab == 'help':
    stdscr.addstr(1, x - 6, "help", curses.color_pair(4))
  else:
    stdscr.addstr(1, x - 6, "help", curses.color_pair(2))
  stdscr.addch(0, x - 12, curses.ACS_TTEE)
  stdscr.addch(1, x - 12, curses.ACS_VLINE)
  stdscr.addch(2, x - 12, curses.ACS_BTEE)

  # Center title
  stdscr.addstr(1, center_x - 4, "EQ ALERT", curses.color_pair(2))


def draw_events_frame(stdscr, char, zone, events):
  """Draw events"""
  y, x = stdscr.getmaxyx()
  center_y = int(y / 2)
  center_x = int(x / 2)

  # Clear and box
  stdscr.clear()
  stdscr.box()

  # Draw tabs
  draw_tabs(stdscr, 'events')

  # Bottom of  events
  stdscr.addch(center_y, 0, curses.ACS_LTEE)
  stdscr.addch(center_y, x - 1, curses.ACS_RTEE)
  for c in range (x - 2):
    stdscr.addch(center_y, c + 1, curses.ACS_HLINE)

  # Character
  stdscr.addstr(center_y + 1, 2, char.title(), curses.color_pair(2))

  # Zone
  stdscr.addstr(center_y + 1, x - len(zone) - 2, zone.title(), curses.color_pair(2))

  # Draw events
  draw_events(stdscr, events)


def draw_events(stdscr, events):
  """Draw events window component of events"""
  y, x = stdscr.getmaxyx()
  center_y = int(y / 2)
  bottom_y = center_y - 4
  top_y = 2

  eventscr = stdscr.derwin(center_y - 3, x - 4, 3, 2)
  eventscr.clear()

  try:
    count = 0
    while count < (bottom_y + 1) and count < len(events):
      event_num = len(events) - count - 1
      event = events[(count * -1) - 1]
      c_y = bottom_y - count
      draw_ftime(eventscr, event.timestamp, c_y)
      eventscr.addch(c_y, 14, curses.ACS_VLINE)
      eventscr.addstr(c_y, 16, str(event.payload), curses.color_pair(1))
      count += 1
  except Exception as e:
      eqa_settings.log('draw events: Error on line ' +
                        str(sys.exc_info()[-1].tb_lineno) + ': '  + str(e))


def draw_ftime(stdscr, timestamp, y):
  """Draw formatted time for events"""
  h, m, second = timestamp.split(":")
  s, ms = second.split('.')

  stdscr.addstr(y, 1, h, curses.color_pair(3))
  stdscr.addstr(y, 3, ':', curses.color_pair(2))
  stdscr.addstr(y, 4, m, curses.color_pair(3))
  stdscr.addstr(y, 6, ':', curses.color_pair(2))
  stdscr.addstr(y, 7, s, curses.color_pair(3))
  stdscr.addstr(y, 9, '.', curses.color_pair(2))
  stdscr.addstr(y, 10, ms, curses.color_pair(3))


def draw_state(stdscr, state, raid):
  """Draw state"""
  y, x = stdscr.getmaxyx()
  center_y = int(y / 2)
  center_x = int(x / 2)

  # Clear and box
  stdscr.clear()
  stdscr.box()

  # Draw tabs
  draw_tabs(stdscr, 'state')

  # Show some state
  try:
    # server state
    stdscr.addstr(5, 5, 'Server', curses.color_pair(2))
    stdscr.addstr(5, 16, ': ', curses.color_pair(1))
    stdscr.addstr(5, 18, state.server.title(), curses.color_pair(3))

    # char
    stdscr.addstr(7, 5, 'Character', curses.color_pair(2))
    stdscr.addstr(7, 16, ': ', curses.color_pair(1))
    stdscr.addstr(7, 18, state.char.title(), curses.color_pair(3))

    # zone
    stdscr.addstr(9, 5, 'Zone', curses.color_pair(2))
    stdscr.addstr(9, 16, ': ', curses.color_pair(1))
    stdscr.addstr(9, 18, state.zone.title(), curses.color_pair(3))

    # loc
    stdscr.addstr(11, 5, 'Location', curses.color_pair(2))
    stdscr.addstr(11, 16, ': ', curses.color_pair(1))
    stdscr.addstr(11, 18, str(state.loc[0]), curses.color_pair(3))
    stdscr.addstr(11, 24, ' : ', curses.color_pair(2))
    stdscr.addstr(11, 26, str(state.loc[1]), curses.color_pair(3))
    stdscr.addstr(11, 32, ' : ', curses.color_pair(2))
    stdscr.addstr(11, 34, str(state.loc[2]), curses.color_pair(3))

    # direction
    stdscr.addstr(13, 5, 'Direction', curses.color_pair(2))
    stdscr.addstr(13, 16, ': ', curses.color_pair(1))
    stdscr.addstr(13, 18, state.direction.title(), curses.color_pair(3))

    # raid state
    stdscr.addstr(15, 5, 'Raid', curses.color_pair(2))
    stdscr.addstr(15, 16, ': ', curses.color_pair(1))
    if not raid.is_set():
      stdscr.addstr(15, 18, 'False', curses.color_pair(3))
    else:
      stdscr.addstr(15, 18, 'True', curses.color_pair(3))

    # afk state
    stdscr.addstr(17, 5, 'AFK', curses.color_pair(2))
    stdscr.addstr(17, 16, ': ', curses.color_pair(1))
    stdscr.addstr(17, 18, state.afk.title(), curses.color_pair(3))

  except Exception as e:
      eqa_settings.log('draw state: Error on line ' +
                        str(sys.exc_info()[-1].tb_lineno) + ': '  + str(e))


def draw_settings(stdscr, state, selected_setting, selected_char):
  """Draw settings"""
  # Clear and box
  stdscr.clear()
  stdscr.box()

  # Draw tabs
  draw_tabs(stdscr, 'settings')

  # Draw chars
  if selected_setting == 'character':
    stdscr.addstr(4, 3, 'Character Selection', curses.A_UNDERLINE | curses.color_pair(2))
  else:
    stdscr.addstr(4, 5, 'Character Selection', curses.color_pair(3))
  stdscr.addstr(12, 5, 'Active Character', curses.color_pair(3))
  stdscr.addstr(12, 21, ':', curses.color_pair(1))
  stdscr.addstr(12, 23, state.char.title(), curses.color_pair(2))

  draw_chars(stdscr, state.chars, state.char, selected_char)


def draw_chars(stdscr, chars, char, selected):
  """Draw character selection component for settings"""
  try:
    y, x = stdscr.getmaxyx()
    charscr_width = int(x / 3)
    charscr_height = 7

    charscr = stdscr.derwin(charscr_height, charscr_width, 5, 3)
    charscr.clear()
    charscr.box()

    count = 0
    if len(chars) < 5:
      while count < len(chars):
        char_name = chars[count]
        if selected == count:
          charscr.addstr(6 - count, 2, char_name.title(), curses.color_pair(1))
        else:
          charscr.addstr(6 - count, 2, char_name.title(), curses.color_pair(2))
        count += 1
    else:
      for count in range(0, 5):
        char_name = chars[(selected - 2 + count) % len(chars)]
        if count == 2:
          charscr.addstr(5 - count, 2, char_name.title(), curses.color_pair(1))
        elif (selected - 2 + count) < len(chars) and (selected - 2 + count) >= 0:
          charscr.addstr(5 - count, 2, char_name.title(), curses.color_pair(2))
        count += 1

  except Exception as e:
      eqa_settings.log('draw chars: Error on line ' +
                       str(sys.exc_info()[-1].tb_lineno) + ': '  + str(e))


def draw_help(stdscr):
  """Draw help"""

  # Clear and box
  stdscr.clear()
  stdscr.box()

  # Draw tabs
  draw_tabs(stdscr, 'help')

  # Commands
  stdscr.addstr(5, 5, "Commands:",curses.color_pair(1))

  # Global commands
  stdscr.addstr(7, 7, "Global",curses.color_pair(1))

  stdscr.addstr(8, 9, "F1", curses.color_pair(2))
  stdscr.addstr(8, 15, ":", curses.color_pair(1))
  stdscr.addstr(8, 17, "Events", curses.color_pair(3))

  stdscr.addstr(9, 9, "F2", curses.color_pair(2))
  stdscr.addstr(9, 15, ":", curses.color_pair(1))
  stdscr.addstr(9, 17, "State", curses.color_pair(3))

  stdscr.addstr(10, 9, "F3", curses.color_pair(2))
  stdscr.addstr(10, 15, ":", curses.color_pair(1))
  stdscr.addstr(10, 17, "Settings", curses.color_pair(3))

  stdscr.addstr(11, 9, "F4", curses.color_pair(2))
  stdscr.addstr(11, 15, ":", curses.color_pair(1))
  stdscr.addstr(11, 17, "Help", curses.color_pair(3))

  stdscr.addstr(12, 9, "q", curses.color_pair(2))
  stdscr.addstr(12, 15, ":", curses.color_pair(1))
  stdscr.addstr(12, 17, "Quit", curses.color_pair(3))

  stdscr.addstr(13, 9, "F12", curses.color_pair(2))
  stdscr.addstr(13, 15, ":", curses.color_pair(1))
  stdscr.addstr(13, 17, "Reload config", curses.color_pair(3))

  # Events commands
  stdscr.addstr(15, 7, "Events",curses.color_pair(1))

  stdscr.addstr(16, 9, "c", curses.color_pair(2))
  stdscr.addstr(16, 15, ":", curses.color_pair(1))
  stdscr.addstr(16, 17, "Clear events", curses.color_pair(3))

  stdscr.addstr(17, 9, "r", curses.color_pair(2))
  stdscr.addstr(17, 15, ":", curses.color_pair(1))
  stdscr.addstr(17, 17, "Toggle raid mode", curses.color_pair(3))

  # Settings commands
  stdscr.addstr(19, 7, "Settings",curses.color_pair(1))

  stdscr.addstr(20, 9, "up", curses.color_pair(2))
  stdscr.addstr(20, 15, ":", curses.color_pair(1))
  stdscr.addstr(20, 17, "Cycle up in selection", curses.color_pair(3))

  stdscr.addstr(21, 9, "down", curses.color_pair(2))
  stdscr.addstr(21, 15, ":", curses.color_pair(1))
  stdscr.addstr(21, 17, "Cycle down in selection", curses.color_pair(3))

  stdscr.addstr(22, 9, "right", curses.color_pair(2))
  stdscr.addstr(22, 15, ":", curses.color_pair(1))
  stdscr.addstr(22, 17, "Toggle selection on", curses.color_pair(3))

  stdscr.addstr(23, 9, "left", curses.color_pair(2))
  stdscr.addstr(23, 15, ":", curses.color_pair(1))
  stdscr.addstr(23, 17, "Toggle selection off", curses.color_pair(3))

  stdscr.addstr(24, 9, "space", curses.color_pair(2))
  stdscr.addstr(24, 15, ":", curses.color_pair(1))
  stdscr.addstr(24, 17, "Cycle selection", curses.color_pair(3))


def draw_toosmall(stdscr):
    """Draw too small warning"""
    stdscr.clear()
    stdscr.box()
    y, x = stdscr.getmaxyx()
    center_y = int(y / 2)
    center_x = int(x / 2)

    stdscr.addstr(center_y, center_x - 10, "Terminal too small.",
        curses.color_pair(1))


if __name__ == '__main__':
  main()



#! /usr/bin/env python

"""
   Program:   EQ Alert
   File Name: eqa_parser.py
   Copyright (C) 2019 Michael Geitz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from collections import deque
import sys
import time

import eqa.lib.eqa_struct as eqa_struct
import eqa.lib.eqa_settings as eqa_settings


def last_line(character_log):
  """Reads and returns last line"""
  try:
    with open(character_log, 'r') as f:
      content = deque(f, 1)
    return content[0]
  except Exception as e:
    eqa_settings.log('last_line: ' + str(e))


def process(exit_flag, log_q, action_q):
  """
    Process: log_q
    Produce action_q
  """

  try:
    while not exit_flag.is_set():
      time.sleep(0.001)
      if not log_q.empty():
        # Read raw log line
        log_line = log_q.get()
        log_q.task_done()

        # Strip line of trailing space and lowercase everything
        line = log_line.strip().lower()
        # Split timestamp and message payload
        timestamp, payload = line[1:].split('] ', 1)
        timestamp = timestamp.split(' ')[3] + '.00'
        # Determine line type
        line_type = determine(payload)
        # Build and queue action
        new_message = eqa_struct.message(timestamp, line_type, 'null', 'null', payload)
        action_q.put(new_message)

  except Exception as e:
      eqa_settings.log('process_log: Error on line ' +
                       str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def determine(line):
    """Determine type of line"""

    line_type = "undetermined"
    line_list = line.split(' ')

   # chat your player initiates
    if line_list[0] == "you":
        if line_list[1] == "told":
            line_type = "you_tell"
        elif line_list[1] == "say,":
            line_type = "you_say"
        elif line_list[1] == "shout,":
            line_type = "you_shout"
        elif "party," in line:
            line_type = "you_group"
        elif line_list[1] == "say":
            if line_list[4] == "character,":
                line_type = "you_ooc"
            if line_list[4] == "guild,":
                line_type = "you_guild"
        elif line_list[1] == "auction,":
            line_type = "you_auction"
            if "wts" in line or "selling" in line and "wtb" in line or "buying" in line:
                line_type = "you_auction_wts:wtb"
            if "wts" in line or "selling" in line:
                line_type = "you_auction_wts"
            if "wtb" in line or "buying" in line:
                line_type = "you_auction_wtb"
        elif line_list[1] == "have":
            if line_list[2] == "entered":
                line_type = "you_new_zone"
            elif line_list[5] == "direction":
                line_type = "direction_miss"
            elif line_list[2] == "healed":
                line_type = "you_healed"
        elif line_list[1] == "think":
            if line_list[4] == "heading":
                line_type = "direction"
        elif "hungry." in line_list:
            line_type = "you_hungry"
        elif line_list[1] == "are":
            if line_list[2] == "now":
                if "a.f.k." in line:
                    line_type = "you_afk_on"
                elif line.endswith("looking for a group."):
                    line_type = "you_lfg_on"
            elif line_list[2] == "no":
                if "a.f.k." in line:
                    line_type = "you_afk_off"
                elif line.endswith("looking for a group."):
                    line_type = "you_lfg_off"
            elif line_list[2] == "out":
                if line_list[-1] == "food." and "drink" not in line and "drink." not in line:
                    line_type = "you_outfood"
                elif line_list[-1] == "drink." and "food" not in line and "food." not in line:
                    line_type = "you_outdrink"
                elif line.endswith("food and drink."):
                    line_type = "you_outfooddrink"
                elif line.endswith("drink and low on food."):
                    line_type = "you_outdrinklowfood"
                elif line.endswith("food and low on drink."):
                    line_type = "you_outfoodlowdrink"
            elif line_list[-1] == "thirsty.":
                line_type = "you_thirsty"
            elif line_list[-1] == "hungry.":
                line_type = "you_hungry"
        elif line_list[1] == "forget":
            line_type == "you_spell_forget"

    elif line_list[0] == "your":
        if line_list[1] == "location":
            if line_list[2] == "is":
                line_type = "location"
        elif line_list[1] == "spell":
            if line_list[3] == "interrupted":
                line_type = "spell_interrupted"
        elif line_list[2] == "spell":
            if line_list[1] == "charm":
                line_type = "spell_break_charm"
            elif line_list[1] == "ensnare":
                line_type = "spell_break_ensare"
            else:
                line_type = "spell_break"
        elif line_list[1] ==  "faction":
            line_type = "faction_line"
        elif line_list[1] == "target":
            if line_list[2] == "resisted":
                line_type = "spell_resist"
            elif line_list[4] == "cured.":
                line_type = "target_cured"
    elif line_list[0].startswith("-"):
        line_type = "who_line"
    elif len(line_list) == 1:
        line_type = "mysterious_oner"

    # chat from other players
    elif line_list[1] == "tells":
        if line_list[2] == "you,":
            line_type = "tell" # or emote
        if line_list[3] == "guild,":
            line_type = "guild" # or emote
        if line_list[3] == "group,":
            line_type = "group" # or emote
    elif line_list[1] == "says,":
        line_type = "say" # or emote
    elif line_list[1] == "shouts,":
        line_type = "shout" # or emote
    elif line_list[1] == "says":
        if line_list[4] == "character,":
            line_type = "ooc" # or emote
    elif line_list[1] == "auctions,":
        line_type = "auction" # or emote
        #if "wts" in line or "selling" in line and "wtb" in line or "buying" in line:
        #    line_type = "auction_wts:wtb"
        if "wts" in line or "selling" in line:
            line_type = "auction_wts"
        elif "wtb" in line or "buying" in line:
            line_type = "auction_wtb"

    # spells / casting
    elif line_list[1] == "spell":
        line_type = "spell_something"
        if line_list[2] == "fizzles!":
            line_type = "spell_fizzle"
    elif line_list[1] == "casting":
        if line_list[3] == "interrupted!":
           line_type = "spell_interrupted"
    elif line_list[1] == "begins":  # assumes only spell messages have [player] begins...
        if len(line_list) > 3:
            if line_list[3] == "regenerate.":
                line_type = "spell_regen"

    # spell damage (has/was)
    elif "has" in line_list:
        if line_list[line_list.index("has") + 1] == "taken" and line_list[line_list.index("has") + 3] == "damage":
            line_type = "dot_damage"
    elif "was" in line_list:
        if line_list[line_list.index("was") + 1] == "hit" and line_list[line_list.index("was") + 3] == "non-melee":
            line_type = "spell_damage"

    # engage messages
    elif "engages" in line_list and line_list[-1].endswith("!"):
        line_type = "engage"

    # combat
    #elif line_list[-2] == "but" and line_list[-1] == "misses!":
    #    line_type = "melee_miss"
    #elif line_list[-1] == "damage.":
    #    if len(line_list) > 2:
    #        if line_list[-3] == "points":
    #            line_type = "melee_hit"

    # other
    # Damn falling messages causing a problem
    #elif "injured" in line:
    #    if line_list[2] == "injured":
    #        if line_list[-1] == "falling.":
    #            line_type == "fall_damage"

    elif line_list[0] == "loading,":
        line_type = "zoning"
    elif line_list[0][0] == "*":
        line_type = "random"
    # chat from in game prompts or commands
    elif line_list[0] == "To" and line_list[1] == "join":
        line_type = "group_invite"
    elif line_list[0] == "there" and line_list[1] == "are":
        line_type = "who_total"
    elif line_list[0].startswith("["):
        line_type = "who_player"
    elif line_list[0] == "afk":
        line_type = "who_player_afk"
    elif line_list[0].startswith("<linedead>"):
        line_type = "who_player_linkdead"
    elif line_list[0] ==  "players" or line_list[0] == "Friends":
        line_type = "who_top"
    elif line_list[0] == "targeted" :
        line_type = "target"

    # possible emotes
    elif line_list[1] == "bows" :
        line_type = "emote_bow"
    elif line_list[1] == "thanks" :
        line_type = "emote_thank"
    elif line_list[1] == "waves" :
        line_type = "emote_wave"
    elif line_list[1] == "dances" :
        line_type = "emote_dance"
    elif line_list[1] == "bonks" :
        line_type = "emote_bonk"
    elif line_list[1] == "beams" :
        line_type = "emote_smile"
    elif line_list[1] == "cheers." or line_list[1] == "cheers":
        line_type = "emote_cheer"

    # other
    elif line_list[0] == "it":
        if line_list[1] == "begins":
            if "rain." in line:
                line_type = "weather_start_rain"
            if "snow." in line:
                line_type = "weather_start_snow"
        elif line_list[1] == "stops":
            if "raining." in line:
                line_type = "weather_stop_rain" # the sky clears as the rain ceases to fall.
            if "snowing." in line:
                line_type = "weather_stop_snow"
        elif line_list[-1] == "camp.":
            line_type == "you_camping"

    return line_type

if __name__ == '__main__':
    main()



#! /usr/bin/env python

"""
   Program:   EQ Alert
   File Name: eqa_config.py
   Copyright (C) 2019 Michael Geitz
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import json
import datetime
import os
import sys

import eqa.lib.eqa_settings as eqa_settings

def init(base_path):
  """If there is no config, make a config"""
  try:
    if not os.path.isfile(base_path + 'config.json'):
      build_config(base_path)

  except Exception as e:
    eqa_settings.log('config init: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def read_config(base_path):
  """read the config"""
  try:
    json_data = open(base_path + 'config.json', 'r')
    config = json.load(json_data)
    json_data.close()

    return config

  except Exception as e:
    eqa_settings.log('config read: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def update_logs(base_path):
  """Add characters and servers of eqemu_ prefixed files in the log path"""
  try:
    json_data = open(base_path + 'config.json', 'r')
    config = json.load(json_data)
    log_files = [ f for f in os.listdir(config["settings"]["paths"]["char_log"]) if os.path.isfile(os.path.join(config["settings"]["paths"]["char_log"],f)) ]

    for logs in log_files:
      if "eqlog_" in logs:
        emu, char_name, end = logs.split("_")
        server_name = end.lower().split('.')[0]
        if char_name.lower() not in config["characters"].keys():
          add_char(char_name.lower(), base_path)
        if server_name not in config["servers"].keys():
          add_server(server_name.lower(), base_path)

    json_data.close()

  except Exception as e:
    eqa_settings.log('set config chars: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def add_char(name, base_path):
  """Adds a new character to the config"""
  try:
    json_data = open(base_path + 'config.json', 'r+')
    data = json.load(json_data)
    data["characters"].update({name.lower():"true"})
    json_data.seek(0)
    json.dump(data, json_data, indent = 4)
    json_data.close()

  except Exception as e:
    eqa_settings.log('add char: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def add_server(server, base_path):
  """Adds a new server to the config"""

  try:
    json_data = open(base_path + 'config.json', 'r+')
    data = json.load(json_data)
    data["servers"].update({server.lower():"true"})
    json_data.seek(0)
    json.dump(data, json_data, indent = 4)
    json_data.close()

  except Exception as e:
    eqa_settings.log('add server: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def get_config_chars(config):
  """Return characters in the config"""
  try:
    chars = []
    for char in config["characters"].keys():
      if char != "default" and not config["characters"][char] == "false":
        chars.append(char)

    return chars

  except Exception as e:
    eqa_settings.log('get chars: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def get_config_servers(config):
  """Return servers from the config"""

  try:
    servers = []
    for server in config["servers"].keys():
      if server != "default" and not config["servers"][server] == "false":
        servers.append(server)

    return servers

  except Exception as e:
    eqa_settings.log('get servers: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def set_default_char(name, base_path):
  """Set a new default character"""

  try:
    json_data = open(base_path + 'config.json', 'r+')
    data = json.load(json_data)
    data["characters"].update({"default":name.lower()})
    json_data.seek(0)
    json.dump(data, json_data, indent = 4)
    json_data.close()

  except Exception as e:
    eqa_settings.log('set default char: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def set_default_server(name, base_path):
  """Set a new default server"""

  try:
    json_data = open(base_path + 'config.json', 'r+')
    data = json.load(json_data)
    data["servers"].update({"default":name.lower()})
    json_data.seek(0)
    json.dump(data, json_data, indent = 4)
    json_data.close()

  except Exception as e:
    eqa_settings.log('set default server: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def add_type(line_type, base_path):
  """Adds default setting values for new line_type"""

  try:
    json_data = open(base_path + 'config.json', 'r+')
    data = json.load(json_data)
    data["line"].update({line_type:{'sound': '0', 'reaction': 'false', 'alert': {}}})
    json_data.seek(0)
    json.dump(data, json_data, indent = 4)
    json_data.close()

  except Exception as e:
    eqa_settings.log('add type: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def add_zone(zone, base_path):
  """Adds default setting values for new zones"""

  try:
    json_data = open(base_path + 'config.json', 'r+')
    data = json.load(json_data)
    data["zones"].update({zone:"false"})
    json_data.seek(0)
    json.dump(data, json_data, indent = 4)
    json_data.close()

  except Exception as e:
    eqa_settings.log('add zone: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))


def build_config(base_path):
  """Build a default config"""

  home = os.path.expanduser("~")

  new_config = """
{
    "zones": {
        "rivervale": "false",
        "the feerrott": "false",
        "mines of nurga": "false",
        "crushbone": "false",
        "lake of ill omen": "false",
        "befallen": "false",
        "everfrost": "false",
        "infected paw": "false",
        "dragon necropolis": "false",
        "high keep": "false",
        "east freeport": "false",
        "lavastorm mountains": "false",
        "east commonlands": "false",
        "great divide": "false",
        "the city of mist": "false",
        "surefall glade": "false",
        "guk": "false",
        "kedge keep": "false",
        "field of bone": "false",
        "west freeport": "false",
        "butcherblock mountains": "false",
        "rathe mountains": "false",
        "the hole": "false",
        "karnor's castle": "false",
        "plane of hate": "raid",
        "kithicor woods": "false",
        "southern felwithe": "false",
        "steamfont mountains": "false",
        "eastern plains of karana": "false",
        "temple of droga": "false",
        "permafrost caverns": "false",
        "paineel": "false",
        "firiona vie": "false",
        "ocean of tears": "false",
        "trakanon's teeth": "false",
        "sleepers tomb": "false",
        "city of thurgadin": "false",
        "oasis of marr": "false",
        "iceclad ocean": "false",
        "north freeport": "false",
        "nagafen's lair": "false",
        "the burning wood": "false",
        "estate of unrest": "false",
        "velketor's labyrinth": "false",
        "warrens": "false",
        "dagnor's cauldron": "false",
        "howling stones": "false",
        "the wakening lands": "false",
        "lesser faydark": "false",
        "the arena": "false",
        "ruins of old guk": "false",
        "the overthere": "false",
        "toxxulia forest": "false",
        "chardok": "false",
        "skyshrine": "false",
        "lost temple of cazic-thule": "false",
        "northern plains of karana": "false",
        "innothule swamp": "false",
        "sirens grotto": "false",
        "plane of fear": "raid",
        "northern felwithe": "false",
        "plane of growth": "false",
        "kael drakkel": "false",
        "western wastelands": "false",
        "skyfire mountains": "false",
        "the nektulos forest": "false",
        "southern desert of ro": "false",
        "crystal caverns": "false",
        "an arena (pvp) area": "false",
        "qeynos hills": "false",
        "south kaladim": "false",
        "the emerald jungle": "false",
        "greater faydark": "false",
        "timorous deep": "false",
        "kurn's tower": "false",
        "icewell keep": "raid",
        "west commonlands": "false",
        "eastern wastelands": "false",
        "cobalt scar": "false",
        "misty thicket": "false",
        "blackburrow": "false",
        "southern plains of karana": "false",
        "erudin": "false",
        "western plains of karana": "false",
        "erudin palace": "false",
        "plane of air": "false",
        "dreadlands": "false",
        "highpass hold": "false",
        "temple of veeshan": "raid",
        "veeshan's peak": "raid",
        "old sebilis": "false",
        "plane of mischief": "false",
        "frontier mountains": "false",
        "castle mistmoore": "false",
        "northern desert of ro": "false",
        "lake rathetear": "false"
    },
    "line": {
        "mysterious_oner": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "all": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_new_zone": {
            "sound": "0",
            "reaction": "all",
            "alert": {}
        },
        "group_invite": {
            "sound": "0",
            "reaction": "speak",
            "alert": {}
        },
        "emote_cheer": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_ooc": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "random": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "auction_wtb": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "melee_miss": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "say": {
            "sound": "0",
            "reaction": "true",
            "alert": {
                "help": "true"
            }
        },
        "spell_break_charm": {
            "sound": "0",
            "reaction": "speak",
            "alert": {}
        },
        "you_tell": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_afk_on": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "auction_wts": {
            "sound": "3",
            "reaction": "true",
            "alert": {
                "spiderling silk": "true"
            }
        },
        "emote_bow": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "group": {
            "sound": "4",
            "reaction": "true",
            "alert": {
                "help": "true",
                "inc": "true",
                "incoming": "true"
            }
        },
        "you_thirsty": {
            "sound": "0",
            "reaction": "speak",
            "alert": {}
        },
        "emote_bonk": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "emote_thank": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "shout": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_outdrinklowfood": {
            "sound": "0",
            "reaction": "speak",
            "alert": {}
        },
        "you_outdrink": {
            "sound": "0",
            "reaction": "speak",
            "alert": {}
        },
        "you_auction_wts": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_healed": {
            "sound": "0",
            "reaction": "all",
            "alert": {}
        },
        "you_say": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "spell_fizzle": {
            "sound": "5",
            "reaction": "true",
            "alert": {}
        },
        "tell": {
            "sound": "0",
            "reaction": "speak",
            "alert": {}
        },
        "emote_smile": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "spell_something": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "spell_interrupted": {
            "sound": "2",
            "reaction": "true",
            "alert": {}
        },
        "ooc": {
            "sound": "1",
            "reaction": "false",
            "alert": {}
        },
        "who_player": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_outfoodlowdrink": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "spell_begin_casting": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "zoning": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_outfooddrink": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "who_player_afk": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "who_line": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "who_total": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "target_cured": {
            "sound": "0",
            "reaction": "speak",
            "alert": {}
        },
        "you_auction_wtb": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "emote_dance": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_lfg_off": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "spell_invis": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "undetermined": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_guild": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_shout": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "engage": {
            "sound": "0",
            "reaction": "speak",
            "alert": {}
        },
        "guild": {
            "sound": "3",
            "reaction": "true",
            "alert": {
                "tash": "raid",
                "fixated": "raid",
                "slow": "raid",
                "rampage": "raid",
                "malo": "raid",
                "fixation": "raid",
                "occlusion": "raid",
                "assist": "raid",
                "sunder": "raid",
                "malosini": "raid"
            }
        },
        "target": {
            "sound": "3",
            "reaction": "false",
            "alert": {}
        },
        "spell_damage": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_hungry": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "dot_damage": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "auction": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_afk_off": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_group": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_auction": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "spell_break_ensare": {
            "sound": "0",
            "reaction": "speak",
            "alert": {}
        },
        "who_top": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "emote_wave": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_outfood": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "you_lfg_on": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "spell_break": {
            "sound": "4",
            "reaction": "false",
            "alert": {}
        },
        "melee_hit": {
            "sound": "2",
            "reaction": "false",
            "alert": {}
        },
        "faction_line": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "spell_resist": {
            "sound": "0",
            "reaction": "speak",
            "alert": {}
        },
        "spell_regen": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "location": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "direction": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        },
        "direction_miss": {
            "sound": "0",
            "reaction": "false",
            "alert": {}
        }
    },
    "characters": {
        "default": "foobar"
    },
    "servers": {
        "default": "project1999",
        "project1999": "true"
    },
    "settings": {
        "paths": {
            "sound": "%ssound/",
            "alert_log": "%slog/",
            "char_log": "%s/.wine/drive_c/Program Files/Sony/EverQuest/Logs/"
        },
        "sounds": {
            "1": "hey.wav",
            "3": "look.wav",
            "2": "listen.wav",
            "5": "hello.wav",
            "4": "watch out.wav"
        }
    }
}
"""

  try:
    f = open(base_path + 'config.json', 'a+')
    f.write(new_config % (base_path, base_path, home))
    f.close()

  except Exception as e:
    eqa_settings.log('build config: Error on line' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))

if __name__ == '__main__':
  main()



from distutils.core import setup

setup(
    name='eqalert',
    version='1.5.0',
    author='Michael Geitz',
    author_email='git@geitz.xyz',
    install_requires=[
        'pyinotify',
        'playsound',
        'gtts',
    ],
    python_requires='>3',
    packages=['eqa', 'eqa.lib'],
    license='LICENSE.txt',
    entry_points={
        "console_scripts": [
            "eqalert = eqa.eqalert:main",
        ],
    },
    description='Parse and React to EQEmu Logs',
    url='https://github.com/mgeitz/eqalert'
)



#! /usr/bin/env python

"""
   Program:   EQ Alert
   File Name: eqalert.py
   Copyright (C) 2019 Michael Geitz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Parse and react to eqemu logs
"""

import pyinotify
import curses
import logging
import datetime
import time
import threading
import queue
import os
import sys

import eqa.lib.eqa_parser as eqa_parse
import eqa.lib.eqa_config as eqa_config
import eqa.lib.eqa_settings as eqa_settings
import eqa.lib.eqa_curses as eqa_curses
import eqa.lib.eqa_sound as eqa_sound
import eqa.lib.eqa_parser as eqa_parser
import eqa.lib.eqa_struct as eqa_struct
import eqa.lib.eqa_action as eqa_action
import eqa.lib.eqa_keys as eqa_keys
import eqa.lib.eqa_state as eqa_state

def bootstrap(base_path):
  """Bootstrap first run"""

  try:
    print("Bootstrapping for first run . . .")

    # Make the main folder
    print("    - putting this stuff in " + base_path)
    os.makedirs(base_path)

    # Make the log folder
    if not os.path.exists(base_path + 'log/'):
      print("    - making a place for logs")
      os.makedirs(base_path + 'log/')

    # Make some sounds
    if not os.path.exists(base_path + 'sound/'):
      print("    - making some sounds")
      os.makedirs(base_path + 'sound/')
      eqa_sound.pre_speak('hello', base_path + 'sound/')
      eqa_sound.pre_speak('hey', base_path + 'sound/')
      eqa_sound.pre_speak('listen', base_path + 'sound/')
      eqa_sound.pre_speak('look', base_path + 'sound/')
      eqa_sound.pre_speak('watch out', base_path + 'sound/')

    # Set default character
      print("    - setting a default character")
    eqa_config.init(base_path)
    eqa_config.update_logs(base_path)
    tmp_config = eqa_config.read_config(base_path)
    tmp_server = tmp_config["servers"]["default"]
    tmp_chars = eqa_config.get_config_chars(tmp_config)
    eqa_config.set_default_char(tmp_chars[0], base_path)

  except Exception as e:
    print('Unfortunately, the bootstrap step failed with: ' +
          str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))
    exit(1)


def main():
  """Main method, does the good stuff"""

  # Paths
  home = os.path.expanduser("~")
  base_path = home + '/.eqa/'

  # Queues
  keyboard_q = queue.Queue()
  action_q = queue.Queue()
  display_q = queue.Queue()
  sound_q = queue.Queue()
  system_q = queue.Queue()
  log_q = queue.Queue()
  heal_q = queue.Queue()
  damage_q = queue.Queue()

  # Bootstraps bootstraps
  if not os.path.exists(base_path):
    bootstrap(base_path)

  # Thread Events
  raid = threading.Event()
  cfg_reload = threading.Event()
  heal_parse = threading.Event()
  spell_parse = threading.Event()
  exit_flag = threading.Event()

  # Build initial state
  logging.basicConfig(filename=base_path + 'log/eqalert.log', level=logging.INFO)
  eqa_config.update_logs(base_path)
  config = eqa_config.read_config(base_path)
  server = config["servers"]["default"]
  chars = eqa_config.get_config_chars(config)
  char = config["characters"]["default"]
  char_log = config["settings"]["paths"]["char_log"] + "eqlog_" + char.title() + '_' + server + ".txt"
  state = eqa_state.EQA_State(char, chars, 'unavailable', [0.00, 0.00, 0.00], 'unavailable', 'false', server)

  # Sanity check
  if not os.path.exists(char_log):
    print('Please review `settings > paths` in config.json, a log with that default server and character cannot be found.')
    exit(1)

  # Turn on the lights
  screen = eqa_curses.init(state)

  ## Consume keyboard events
  ## Produce keyoard_q
  read_keys       = threading.Thread(target=eqa_keys.read,
       args       = (exit_flag, keyboard_q, screen))
  read_keys.daemon = True
  read_keys.start()

  ## Process log_q
  ## Produce action_q
  process_log     = threading.Thread(target=eqa_parser.process,
            args  = (exit_flag, log_q, action_q))
  process_log.daemon = True
  process_log.start()

  ## Process keyboard_q
  ## Produce display_q, sound_q, system_q
  process_keys    = threading.Thread(target=eqa_keys.process,
            args  = (keyboard_q, system_q, display_q, sound_q,
                      exit_flag, heal_parse, spell_parse, raid, chars))
  process_keys.daemon = True
  process_keys.start()

  ## Consume action_q
  ## Produce display_q, sound_q, system_q, heal_q, damage_q
  process_action  = threading.Thread(target=eqa_action.process,
            args  = (action_q, system_q, display_q, sound_q, heal_q, damage_q,
                      exit_flag, heal_parse, spell_parse, raid, cfg_reload,
                      config, base_path))
  process_action.daemon = True
  process_action.start()

  ## Consume sound_q
  process_sound   = threading.Thread(target=eqa_sound.process,
            args  = (config, sound_q, exit_flag, cfg_reload))
  process_sound.daemon = True
  process_sound.start()

  ## Consume display_q
  process_display = threading.Thread(target=eqa_curses.display,
            args  = (screen, display_q, state, raid, exit_flag))
  process_display.daemon = True
  process_display.start()

  ## Consume char_log
  ## Produce log_q

  def callback(event):
    try:
      if event.mask == pyinotify.IN_CLOSE_WRITE:
        log_q.put(eqa_parser.last_line(char_log))
    except Exception as e:
      eqa_settings.log('log watch callback: Error on line ' +
                        str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))

  try:
    log_watch = pyinotify.WatchManager()
    log_watch.add_watch(char_log, pyinotify.IN_CLOSE_WRITE, callback)
    log_notifier = pyinotify.ThreadedNotifier(log_watch)
    log_notifier.daemon = True
    log_notifier.start()
  except Exception as e:
    eqa_settings.log('log watch callback: Error on line ' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))

  # And we're on
  display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'draw', 'events', 'null'))
  display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', 'Initialized'))
  sound_q.put(eqa_struct.sound('espeak', 'initialized'))

  ## Consume system_q
  try:
    while not exit_flag.is_set():
      time.sleep(0.001)
      if not system_q.empty():
        new_message = system_q.get()
        system_q.task_done()

        if new_message.type == "system":
          # Update zone
          if new_message.tx == "zone":
            state.set_zone(new_message.payload)
          # Update afk status
          elif new_message.tx == "afk":
            state.set_afk(new_message.payload)
          # Update location
          elif new_message.tx == "loc":
            state.set_loc(new_message.payload)
          # Update direction
          elif new_message.tx == "direction":
            state.set_direction(new_message.payload)
          # Update server
          elif new_message.tx == "server":
            state.set_server(new_message.payload)
          # Update character
          elif new_message.tx == "new_character" and not new_message.payload == state.char:
            new_char_log = config["settings"]["paths"]["char_log"] + "eqlog_" + new_message.payload.title() + '_' + state.server + ".txt"
            # Ensure char/server combo exists as file
            if os.path.exists(new_char_log):
              # Stop watch on current log
              log_watch.rm_watch(char_log, pyinotify.IN_CLOSE_WRITE, callback)
              # Set new character
              state.set_char(new_message.payload)
              char_log = new_char_log
              # Start new log watch
              log_watch.add_watch(char_log, pyinotify.IN_CLOSE_WRITE, callback)
              display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', "Character changed to " + state.char))
              sound_q.put(eqa_struct.sound('espeak', 'Character changed to ' + state.char))
            else:
              display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', "Unable to change characters, please review logs"))
              eqa_settings.log('Could not find file: ' + new_char_log)
          # Reload config
          elif new_message.tx == "reload_config":
            # Reload config
            eqa_config.update_logs(base_path)
            config = eqa_config.read_config(base_path)
            # Reread characters
            state.set_chars(eqa_config.get_config_chars(config))
            # Stop process_action and process_sound
            cfg_reload.set()
            process_action.join()
            process_sound.join()
            cfg_reload.clear()
            # Start process_action and process_sound
            process_action  = threading.Thread(target=eqa_action.process,
                      args  = (action_q, system_q, display_q, sound_q, heal_q, damage_q,
                                exit_flag, heal_parse, spell_parse, raid, cfg_reload,
                                config, base_path))
            process_action.daemon = True
            process_action.start()
            process_sound  = threading.Thread(target=eqa_sound.process,
                      args = (config, sound_q, exit_flag, cfg_reload))
            process_sound.daemon = True
            process_sound.start()
            display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', 'Configuration reloaded'))
            sound_q.put(eqa_struct.sound('espeak', 'Configuration reloaded'))
        else:
          display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', new_message.type + ': ' + new_message.payload))

  except Exception as e:
    eqa_settings.log('main: Error on line ' +
                      str(sys.exc_info()[-1].tb_lineno) + ': ' + str(e))
    pass

  # Exit
  display_q.put(eqa_struct.display(eqa_settings.eqa_time(), 'event', 'events', 'Exiting'))
  read_keys.join()
  process_log.join()
  process_keys.join()
  process_action.join()
  process_sound.join()
  process_display.join()
  log_watch.rm_watch(char_log, pyinotify.IN_CLOSE_WRITE, callback)
  log_notifier.stop()
  eqa_curses.close_screens(screen)


if __name__ == '__main__':
  main()
